import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  max,
  format,
  extent
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { DropDown } from './DropDown';

const width = 960;
const menuHeight = 50;
const height = 500 - menuHeight;
const margin = {
  top: 20,
  right: 20,
  bottom: 60,
  left: 80,
};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 40;

const attributes = [
    { value: 'sepal_length', label: 'Sepal Length' }, 
    { value: 'sepal_width', label: 'Sepal Width' }, 
    { value: 'petal_length', label: 'Petal Length' }, 
    { value: 'petal_width', label: 'Petal Width' }, 
    { value: 'species', label: 'Species' }
  ];

const getLabel = value => {
	for(let i = 0; i < attributes.length; i++){
  	if (attributes[i].value === value){
				return attributes[i].label;
		}
  }
};

const App = () => {
  const data = useData();

  const initialXAttribute = 'sepal_length';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = (d) => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);
  
  const initialYAttribute = 'sepal_width';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = (d) => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);
  
  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.right - margin.left;
 
  const circleRadius = 8;
  
  //const yValue = (d) => d.sepal_width;
  //const yAxisLabel = 'Sepal Width';
 
  const siFormat = format('.2s');
  
  const xAxisTickFormat = (tickValue) =>
    siFormat(tickValue).replace('G', 'B');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
  	.nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <>
      <label for="x-select">X</label>
      <DropDown 
      selectedValue={xAttribute}
      options={attributes} 
      id='x-select' 
      onSelectedValueChange={setXAttribute} />
      
      <label for="y-select">Y</label>
      <DropDown 
      selectedValue={yAttribute}
      options={attributes} 
      id='y-select' 
      onSelectedValueChange={setYAttribute} />
      
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={5}
        />
        <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={5}/>
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <text
          className="axis-label"
          
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},
          ${innerHeight / 2}) rotate(-90) `}
        >
          {yAxisLabel}
        </text>
        <Marks
          data={data}
          yScale={yScale}
          xScale={xScale}
          yValue={yValue}
          xValue={xValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={circleRadius}
        />
      </g>
    </svg>
      </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
